;;; thingatpt-transform-delimited.el - parentized into bracketed etc.
;; Copyright (C) 2014 Andreas Roehler, unless
;; indicated otherwise

;; Author: Andreas Roehler <andreas.roehler@easy-emacs.de>, unless
;; indicated otherwise

;; Keywords: convenience

;; This file is free software; you can redistribute it
;; and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2, or (at your option) any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Commentary:

(defun ar-parentized2bracketed-atpt ()
  "Replace an expression (foo) by [foo]"
  (interactive "*")
  (save-excursion
    (let* ((bounds (ignore-errors (ar-bounds-of-parentized-atpt)))
           (end (copy-marker (cdr bounds))))
      (if bounds
	  (progn
	    (ar-trim-parentized-atpt)
	    (goto-char (car bounds))
	    (insert "[")
	    (goto-char end)
	    (insert "]"))
        (message "Can't see %s at point" "\"parentized\"")))))

(defun ar-parentized2braced-atpt ()
  "Replace an expression (foo) by {foo}"
  (interactive "*")
  (save-excursion
    (let* ((bounds (ar-bounds-of-parentized-atpt))
           (end (copy-marker (cdr bounds))))
      (if bounds
	  (progn
	    (ar-trim-parentized-atpt)
	    (goto-char (car bounds))
	    (insert "{")
	    (goto-char end)
	    (insert "}"))
        (message "Can't see %s at point" "\"parentized\"")))))

(defun ar-bracketed2braced-atpt ()
  "Replace an expression [foo] by {foo}"
  (interactive "*")
  (save-excursion
    (let* ((bounds (ignore-errors (ar-bounds-of-bracketed-atpt)))
           (end (copy-marker (cdr bounds))))
      (if bounds
	  (progn
	    (ar-trim-bracketed-atpt)
	    (goto-char (car bounds))
	    (insert "{")
	    (goto-char end)
	    (insert "}"))
        (message "Can't see %s at point" "\"bracketed\"")))))

(defun ar-bracketed2parentized-atpt ()
  (interactive "*")
  "Replace an expression [foo] by (foo)"
  (save-excursion
    (let* ((bounds (ignore-errors (ar-bounds-of-bracketed-atpt)))
           (end (copy-marker (cdr bounds))))
      (if bounds
	  (progn
	    (ar-trim-bracketed-atpt)
	    (goto-char (car bounds))
	    (insert "(")
	    (goto-char end)
	    (insert ")"))
        (message "Can't see %s at point" "\"bracketed\"")))))

(defun ar-braced2bracketed-atpt ()
  "Replace an expression {foo} by [foo]"
  (interactive "*")
  (save-excursion
    (let* ((bounds (ignore-errors (ar-bounds-of-braced-atpt)))
           (end (copy-marker (cdr bounds))))
      (if bounds
	  (progn
	    (ar-trim-braced-atpt)
	    (goto-char (car bounds))
	    (insert "[")
	    (goto-char end)
	    (insert "]"))
        (message "Can't see %s at point" "\"braced\"")))))

{defun ar-braced2parentized-atpt ()
  "Replace an expression {foo} by (foo)"
  (interactive "*")
  (save-excursion
    (let* ((bounds (ignore-errors (ar-bounds-of-braced-atpt)))
           (end (copy-marker (cdr bounds))))
      (if bounds
	  (progn
	    (ar-trim-braced-atpt)
	    (goto-char (car bounds))
	    (insert "(")
	    (goto-char end)
	    (insert ")"))
        (message "Can't see %s at point" "\"braced\""))))}

(provide 'thingatpt-transform-delimited)
;;; thingatpt-transform-delimited.el ends here
